/*
 * SPDX-FileCopyrightText: 2014 Hugo Pereira Da Costa <hugo.pereira@free.fr>
 *
 * SPDX-License-Identifier: GPL-2.0-or-later
 */

#pragma once

namespace Breeze
{
struct PropertyNames {
    static const char noAnimations[];
    static const char noWindowGrab[];
    static const char netWMForceShadow[];
    static const char netWMSkipShadow[];
    static const char sidePanelView[];
    static const char toolButtonAlignment[];
    static const char alteredBackground[];
    static const char highlightNeutral[];
    static const char noSeparator[];
    static const char isTopMenu[];
    static const char menuSeamlessEdges[];
    static const char bordersSides[];
    static const char forceFrame[];
    static const char statusBarSeparator[];
    static const char roundButton[];
};

}
